/*
 * Carbon Mini Application
 *
 * This is sample application for debugging and learning purposes only.
 */

#include <unistd.h>
#include <Carbon/Carbon.h>

CGContextRef myContext;
WindowRef myWindow;


void MyCreateAndDrawBitmapImage ()
{ 
  CFBundleRef myBundle;
  CGImageRef image; 
  CGDataProviderRef provider; 
  CFURLRef url; 
  CGRect myRect;

  myBundle = CFBundleGetMainBundle();
  url = CFBundleCopyResourceURL( myBundle,
  				 CFSTR("ooonologointro"),
				 CFSTR("jpg"),
				 NULL ); 
  provider = CGDataProviderCreateWithURL (url);
  CFRelease (url);
  image = CGImageCreateWithJPEGDataProvider (provider, 
					     NULL, 
					     true, 
					     kCGRenderingIntentDefault);
  fprintf(stderr, "Image width: %ld\n", CGImageGetWidth(image));
  fprintf(stderr, "Image height: %ld\n", CGImageGetHeight(image));
  myRect = CGRectMake (0, 0, CGImageGetWidth(image), CGImageGetHeight(image) );
  CGDataProviderRelease (provider);
  CGContextDrawImage (myContext, myRect, image);
  CGImageRelease (image);

  CGContextSetRGBFillColor (myContext, 0, 0, 0, 1);

  myRect = CGRectMake (10, 20, 420, 5);
  CGContextFillRect(myContext, myRect);
} 

OSStatus MyWindowEventHandler (EventHandlerCallRef myHandler, EventRef theEvent, void* userData)
{
  OSStatus result = eventNotHandledErr;

  switch (GetEventKind (theEvent))
    {
    case kEventWindowClose:
 
      fprintf(stderr, "Closing application.\n");
      QuitApplicationEventLoop();
      result = noErr;
      break;
 
    case kEventWindowBoundsChanged:
 
      fprintf(stderr, "Window bounds changed.\n");
      result = noErr;
      break;

    case kEventWindowDrawContent:

      MyCreateAndDrawBitmapImage (myContext);
 
      fprintf(stderr, "Window draw content event happened.\n");
      result = noErr;
      break;
    }

  return (result);
}

void CreateMyWindow() {

  Rect contentBounds;

  CFStringRef windowTitle;
  WindowClass windowClass = kDocumentWindowClass;
  WindowAttributes attributes =
    kWindowStandardDocumentAttributes |
    kWindowStandardHandlerAttribute;

  EventTypeSpec eventList[] = {
    { kEventClassWindow, kEventWindowClose },
    { kEventClassWindow, kEventWindowDrawContent },
    { kEventClassWindow, kEventWindowBoundsChanged },
  };   

  windowTitle = CFCopyLocalizedStringFromTable(CFSTR("Carbon Mini Application"),
					       CFSTR("CarbonMiniApp"),
					       CFSTR("The title of the application."));

  SetRect(&contentBounds, 100, 100, 100+440, 100+286);

  CreateNewWindow (windowClass,
		   attributes,
		   &contentBounds,
		   &myWindow);
  // FIXME: how to get Graphic Context easily without QuickDraw or getting it from the Event?
  QDBeginCGContext (GetWindowPort (myWindow), &myContext);

  SetWindowTitleWithCFString (myWindow, windowTitle);
  CFRelease(windowTitle);

  InstallWindowEventHandler(myWindow,
			    NewEventHandlerUPP (MyWindowEventHandler),
			    GetEventTypeCount(eventList),
			    eventList, NULL, NULL);

  ShowWindow(myWindow);
}

int main () {

  CreateMyWindow();
  RunApplicationEventLoop();

  return 0;
}
